import {
  Select,
  ModalService,
  Modal,
  GridEditable,
  Radio,
  Button,
  Form,
  IButtonEvent,
} from '@zeedhi/common';
import {
  IDictionary,
  Metadata,
  IEventParam,
  Loader,
  Config,
  Http,
  Mask,
} from '@zeedhi/core';
import { PmeMask } from '@/config';
import { ZdRadio } from '@zeedhi/vuetify';

export class ImportacaoReceitaController {
  private appController = Loader.getInstance('AppController');

  private utilities = Loader.getInstance('Utilities');

  private receitaController = Loader.getInstance('ReceitaController');

  private gridEditController = Loader.getInstance('GridEditableController');

  private modalImport!: Modal;

  private recipeDest = '';

  private prodDest: IDictionary= [];

  public get nmPlate() {
    const plate = this.receitaController.selectedPlate;
    const cdprato = Mask.getValueWithMask(plate.CDPRATO, this.utilities.maskPlate());
    return `${cdprato} - ${plate.NMPRATO}`;
  }
  
  public async createImportModal() {
    this.receitaController.setTabPlate();
    this.modalImport = await ModalService.createFromJson('/modals/importacaoReceita', true);
  }

  public showModalImport({ component }: IDictionary) {
    this.modalImport.show();
    const name = component.name;
    if (name.includes('PADRAO') > 0) this.recipeDest = 'PADRAO';
    else if (name.includes('TIPOCARD') > 0) this.recipeDest = 'TIPOCARD';
    else if (name.includes('LOCAL') > 0) this.recipeDest = 'LOCAL';
    else this.recipeDest = 'SERVICO';

    const gridDest = this.utilities.returnGrid(component) as GridEditable;
    this.prodDest = gridDest.datasource.data;
  }

  public destroyModalImport() {
    this.modalImport.destroy();
  }
  
  public closeModalImport() {
    this.modalImport.hide();
  }

  public fullscreen({ component }: IButtonEvent) {
    const modal: IDictionary = Metadata.getInstance('modalHeaderImportacao');
    modal.parent.fullscreen = !modal.parent.fullscreen;
    component.iconName = (modal.parent.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }

  public fieldDefaultValueImport({ component }: IEventParam<Select>) {
    if (component.name === 'TPCARD') {
      component.datasource.data = [this.receitaController.tpCard];
      component.setValue(this.receitaController.tpCard);
    } else if (component.name === 'CDFILIAL') {
      component.datasource.data = [this.receitaController.branch];
      component.setValue(this.receitaController.branch.CDFILIAL);
    } else if (component.name === 'CDSERVFILI') {
      component.datasource.data = [this.receitaController.service];
      component.setValue(this.receitaController.service.CDSERVFILI);
    } else if (component.name === 'CDPRATO') {
      component.datasource.data = [this.receitaController.selectedPlate];
      component.setValue(this.receitaController.selectedPlate);
    }

    if (component.name === 'CENTRAL') {
      component.datasource.get().then((response: IDictionary) => {
        if (response.length > 0) {
          component.setValue(this.appController.getCdCentral());
        }
      });
    }
  }

  public enableFieldsImport({ component }: IEventParam<ZdRadio>) {
    const tpCard = Metadata.getInstance<Select>('TPCARD');
    const cdfilial = Metadata.getInstance<Select>('CDFILIAL');
    const cdservfili = Metadata.getInstance<Select>('CDSERVFILI');
    if (component.value === 'PADRAO') {
      tpCard.disabled = true;
      cdfilial.disabled = true;
      cdservfili.disabled = true;
    } else if (component.value === 'TIPOCARD') {
      tpCard.disabled = false;
      cdfilial.disabled = true;
      cdservfili.disabled = true;
    } else if (component.value === 'LOCAL') {
      tpCard.disabled = true;
      cdfilial.disabled = false;
      cdservfili.disabled = true;
    } else {
      tpCard.disabled = true;
      cdfilial.disabled = false;
      cdservfili.disabled = false;
    }

    this.setFilterGrid();
  }

  public setPratoFilter({ component }: IEventParam<Select>) {
    component.datasource.filter = {
      NIVEL: (Config.masks.plateMask as PmeMask).nivelMax,
    };
    component.datasource.get();
  }

  public fieldChangeFilterGrid({ component }: IEventParam<Select>) {
    if (!component.disabled) {
      this.setFilterGrid();
    }
  }

  public setFilterGrid() {
    const type = Metadata.getInstance<Radio>('TYPE');
    const central = Metadata.getInstance<Select>('CENTRAL');
    const tpCard = Metadata.getInstance<Select>('TPCARD');
    const cdfilial = Metadata.getInstance<Select>('CDFILIAL');
    const cdservfili = Metadata.getInstance<Select>('CDSERVFILI');
  
    const grid: GridEditable = Metadata.getInstance('RECIPE');
    
    let conditions = [];
    
    if (this.formIsValid(type.value)) {
      conditions = [
        {
          name: 'TIPO',
          operator: '=',
          value: type.value || null,
        },
        {
          name: 'CDPRATO',
          operator: '=',
          value: this.receitaController.selectedPlate.CDPRATO,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: this.receitaController.selectedPlate.CDFILAUXPRAT,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: central.value || null,
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: tpCard.value ? tpCard.value.CDTIPOCARD : null,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: cdfilial.value ? cdfilial.value.CDFILIAL : null,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: cdservfili.value ? cdservfili.value.CDSERVFILI : null,
        },
      ];
    } else {
      conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: null,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: null,
        },
      ];
    }
    
    grid.datasource.filter = conditions;
    grid.datasource.get();
  }

  public clearDependentField({ component }: IDictionary) {
    if (component.value) {
      const tpCard: Select = Metadata.getInstance('TPCARD');
      const cdfilial: Select = Metadata.getInstance('CDFILIAL');
      const cdservfili: Select = Metadata.getInstance('CDSERVFILI');
      if (component.name === 'CENTRAL') {
        tpCard.setValue(null);
        cdfilial.setValue(null);
        cdservfili.setValue(null);
      } else {
        cdservfili.setValue(null);
      }

      this.setFilterGrid();
    }
  }

  public calculateFieldsImport({ row, component }: IDictionary) {
    // if (!row.ORIGINAL) row.ORIGINAL = { ...row };
    row.EDITED = true;
    const nameField = component.name.split('_')[0];
    if (!row.OLDROW) row.OLDROW = { ...row };
    row[nameField] = component.value;

    if (nameField !== 'VRPEFATCOC') row.VRPEFATCOC = row.VRPEFATCOCAP - 100; // ganho coccao
    if (nameField !== 'VRPEFATCOCAP') row.VRPEFATCOCAP = 100 + row.VRPEFATCOC; // aprov. coccao
    
    if (nameField !== 'VRPEFATCOR') row.VRPEFATCOR = 100 - row.VRPEFATCORAP; // perda limpeza
    if (nameField !== 'VRPEFATCORAP') row.VRPEFATCORAP = 100 - row.VRPEFATCOR; // aprov. limpeza

    const qtProd = row.QTPRODPRAT;
    const qtBruta = row.QTBRUTA;

    if (!this.receitaController.idqtliqubrut && nameField !== 'QTBRUTA') { // qtd bruta
      row.QTBRUTA = qtProd - ((qtProd * row.VRPEFATCOR) / 100) + ((qtProd * row.VRPEFATCOC) / 100);
    }
    if (this.receitaController.idqtliqubrut && nameField !== 'QTPRODPRAT') { // quantidade
      row.QTPRODPRAT = qtBruta - ((qtBruta * row.VRPEFATCOR) / 100) + ((qtBruta * row.VRPEFATCOC) / 100);
    }

    row.QTCALCNUTR = ((row.VRPEFATNUT * row.QTPRODPRAT) / 100);

    row.QTUNIDEST = row.VRFATOCONVES ? row.QTBRUTA * row.VRFATOCONVES : row.QTBRUTA;
    
    row.QTBRUTAUNR = row.QTUNIDEST;

    const grid = Metadata.getInstance<GridEditable>('RECIPE');
    grid.updateRow(row);
  }

  public saveImport() {
    const grid = Metadata.getInstance<GridEditable>('RECIPE');
    const type = Metadata.getInstance<Radio>('TYPE');
    const form = Metadata.getInstance<Form>('formImport');
    
    if (this.formIsValid(type.value)) {
      const rows: IDictionary = grid.datasource.data;
      
      if (rows.length > 0) {
        const importType = Metadata.getInstance<Radio>('IMPORTTYPE');
        const filter: IDictionary = {};
        let validHour = true;

        filter.CDPRATO = form.value.CDPRATO?.CDPRATO || null;
        filter.CDFILAUXPRAT = form.value.CDPRATO?.CDFILAUXPRAT || null;
        filter.CDFILCENTPME = form.value.CENTRAL || null;
        filter.CDTIPOCARD = form.value.TPCARD?.CDTIPOCARD || null;
        filter.CDFILIAL = form.value.CDFILIAL?.CDFILIAL || null;
        filter.CDSERVFILI = form.value.CDSERVFILI?.CDSERVFILI || null;

        rows.forEach((e: IDictionary) => {
          e.IMPORTTYPE = importType.value;
          e.RECIPEDEST = this.recipeDest;
          e.TYPE = type.value;
          e.CDTIPOCARD = this.receitaController.tpCard.CDTIPOCARD;
          e.CDFILIAL = this.receitaController.branch.CDFILIAL;
          e.CDSERVFILI = this.receitaController.service.CDSERVFILI;
          e.CDPRATO = this.receitaController.selectedPlate.CDPRATO;
          e.CDFILAUXPRAT = this.receitaController.selectedPlate.CDFILAUXPRAT;
          e.CDFILCENTPME = this.appController.getCdCentral();
          e.ORIGEM = filter;
          if (e.HRTEMPPREP && !this.utilities.timeIsValid(e.HRTEMPPREP)) validHour = false;
          e.HRTEMPPREP = this.utilities.formatHour(e.HRTEMPPREP, false);
        });

        const params = [rows, this.prodDest];

        if (!validHour) this.utilities.showAlert('alertHour', 'pme::horaInvalida', 'warning');
        else this.callSave('/saveImportRecipe', params, grid);
      } else this.utilities.showAlert('alertWarning', 'pme::pratoSemReceita', 'warning');
    } else form.validate();
  }

  private callSave(url: string, params: IDictionary, grid: GridEditable) {
    const button = Metadata.getInstance<Button>('saveImportButton');
    this.utilities.activeLoadingButton(button);
    Http.post(url, { requestType: 'Row', row: params }).then((response) => {
      if (response) {
        // this.editable = false;
        this.utilities.showAlert('alertSuccess', 'pme::importadoComSucesso', 'green');
        if (grid.editing) this.gridEditController.defaultState(grid);
        this.closeModalImport();
        let gridDest: GridEditable;
        if (this.recipeDest === 'PADRAO') gridDest = Metadata.getInstance<GridEditable>('RECIPE_STANDARD');
        else if (this.recipeDest === 'TIPOCARD') gridDest = Metadata.getInstance<GridEditable>('RECIPE_TIPOCARD');
        else if (this.recipeDest === 'LOCAL') gridDest = Metadata.getInstance<GridEditable>('RECIPE_LOCAL');
        else gridDest = Metadata.getInstance<GridEditable>('RECIPE_SERVICE');

        gridDest.datasource.get();
        this.receitaController.callEventRecipeChange();
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      this.utilities.disableLoadingButton(button);
    });
  }

  private formIsValid(type: string) {
    let valid = true;
    const form = Metadata.getInstance<Form>('formImport');

    if (type === 'PADRAO') {
      if (!form.value.CENTRAL) valid = false;
    } else if (type === 'TIPOCARD') {
      if (!form.value.CENTRAL || !form.value.TPCARD) valid = false;
    } else if (type === 'LOCAL') {
      if (!form.value.CENTRAL || !form.value.CDFILIAL) valid = false;
    } else if (!form.value.CENTRAL || !form.value.CDFILIAL || !form.value.CDSERVFILI) valid = false;

    return valid;
  }

  public setFilterFilial() {
    const cdfilial: Select = Metadata.getInstance('CDFILIAL');
    const central: Select = Metadata.getInstance('CENTRAL');

    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central.value || null,
      },
    ];
    cdfilial.datasource.filter = conditions;
    cdfilial.datasource.get();
  }
  
  public setFilterService() {
    const cdservfili: Select = Metadata.getInstance('CDSERVFILI');
    const cdfilial: Select = Metadata.getInstance('CDFILIAL');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial.value?.CDFILIAL || null,
      },
    ];
    cdservfili.datasource.filter = conditions;
    cdservfili.datasource.get();
  }

  public setFilterTpCard() {
    const tpCard: Select = Metadata.getInstance('TPCARD');
    const central: Select = Metadata.getInstance('CENTRAL');
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central.value || null,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    tpCard.datasource.filter = conditions;
    tpCard.datasource.get();
  }
}
